'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);
var reactNative = require('react-native');

var ClippedFragment = function ClippedFragment(props) {
  var children = props.children,
      _props$move = props.move,
      move = _props$move === void 0 ? false : _props$move,
      _props$debug = props.debug,
      debug = _props$debug === void 0 ? false : _props$debug,
      width = props.width,
      height = props.height,
      originalWidth = props.originalWidth,
      originalHeight = props.originalHeight,
      _props$left = props.left,
      left = _props$left === void 0 ? 0 : _props$left,
      _props$top = props.top,
      top = _props$top === void 0 ? 0 : _props$top,
      translateX = props.translateX,
      translateY = props.translateY,
      scaleX = props.scaleX,
      scaleY = props.scaleY,
      rotateZ = props.rotateZ,
      rotateX = props.rotateX,
      rotateY = props.rotateY,
      opacity = props.opacity,
      overlayColor = props.overlayColor,
      overlayOpacity = props.overlayOpacity,
      perspective = props.perspective;
  var outerStyle = {
    position: 'absolute',
    left: left,
    top: top,
    width: width,
    height: height,
    overflow: 'hidden',
    transform: []
  };
  var innerStyle = {
    position: 'absolute',
    left: -left,
    top: -top,
    width: originalWidth === undefined ? width : originalWidth,
    height: originalHeight === undefined ? height : originalHeight,
    transform: []
  };

  if (perspective) {
    outerStyle.transform.push({
      perspective: perspective
    });
  }

  if (rotateZ !== undefined) {
    innerStyle.transform.push({
      rotateZ: "-".concat(rotateZ).replace('--', '')
    });
  }

  if (translateX !== undefined) {
    outerStyle.transform.push({
      translateX: translateX
    });
    if (!move) innerStyle.transform.push({
      translateX: reactNative.Animated.subtract(0, translateX)
    });
  }

  if (translateY !== undefined) {
    outerStyle.transform.push({
      translateY: translateY
    });
    if (!move) innerStyle.transform.push({
      translateY: reactNative.Animated.subtract(0, translateY)
    });
  }

  if (scaleX !== undefined) {
    outerStyle.transform.push({
      scaleX: scaleX
    });
    if (!move) innerStyle.transform.push({
      scaleX: reactNative.Animated.divide(1, scaleX)
    });
  }

  if (scaleY !== undefined) {
    outerStyle.transform.push({
      scaleY: scaleY
    });
    if (!move) innerStyle.transform.push({
      scaleY: reactNative.Animated.divide(1, scaleY)
    });
  }

  if (rotateX !== undefined) {
    outerStyle.transform.push({
      rotateX: rotateX
    });
    outerStyle.backfaceVisibility = 'hidden';
  }

  if (rotateY !== undefined) {
    outerStyle.transform.push({
      rotateY: rotateY
    });
    outerStyle.backfaceVisibility = 'hidden';
  }

  if (rotateZ !== undefined) {
    outerStyle.transform.push({
      rotateZ: rotateZ
    });
  }

  if (debug) {
    outerStyle.overflow = 'visible';
    outerStyle.backgroundColor = 'rgba(0, 0, 255, 0.1)';
    outerStyle.borderColor = 'royalblue';
    outerStyle.borderWidth = 1;
    outerStyle.borderStyle = 'dashed';
    return React__default.createElement(reactNative.Animated.View, {
      style: outerStyle
    }, React__default.createElement(reactNative.Animated.View, {
      style: innerStyle
    }, children));
  }

  if (opacity !== undefined) {
    outerStyle.opacity = opacity;
  }

  var overlay;

  if (overlayColor) {
    overlay = React__default.createElement(reactNative.Animated.View, {
      style: Object.assign({}, reactNative.StyleSheet.absoluteFillObject, {
        backgroundColor: overlayColor,
        opacity: overlayOpacity || 1
      })
    });
  }

  return React__default.createElement(reactNative.Animated.View, {
    style: outerStyle
  }, React__default.createElement(reactNative.Animated.View, {
    style: innerStyle
  }, children, overlay));
};

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _inherits(subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function");
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      writable: true,
      configurable: true
    }
  });
  if (superClass) _setPrototypeOf(subClass, superClass);
}

function _getPrototypeOf(o) {
  _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {
    return o.__proto__ || Object.getPrototypeOf(o);
  };
  return _getPrototypeOf(o);
}

function _setPrototypeOf(o, p) {
  _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
    o.__proto__ = p;
    return o;
  };

  return _setPrototypeOf(o, p);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function _objectWithoutProperties(source, excluded) {
  if (source == null) return {};

  var target = _objectWithoutPropertiesLoose(source, excluded);

  var key, i;

  if (Object.getOwnPropertySymbols) {
    var sourceSymbolKeys = Object.getOwnPropertySymbols(source);

    for (i = 0; i < sourceSymbolKeys.length; i++) {
      key = sourceSymbolKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue;
      target[key] = source[key];
    }
  }

  return target;
}

function _assertThisInitialized(self) {
  if (self === void 0) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return self;
}

function _possibleConstructorReturn(self, call) {
  if (call && (typeof call === "object" || typeof call === "function")) {
    return call;
  }

  return _assertThisInitialized(self);
}

function _toConsumableArray(arr) {
  return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread();
}

function _arrayWithoutHoles(arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  }
}

function _iterableToArray(iter) {
  if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter);
}

function _nonIterableSpread() {
  throw new TypeError("Invalid attempt to spread non-iterable instance");
}

/*
function arc(animValue: any): any {
  return Animated.multiply(animValue, animValue);
}

function arc2(animValue: any): any {
  const sub = Animated.subtract(1, animValue);
  return Animated.subtract(1, Animated.multiply(sub, sub));
}*/
var ClippedAnimations = {
  slideInLeft: [{
    translateX: [1, 0]
  }],
  slideInRight: [{
    translateX: [-1, 0]
  }],
  slideInUp: [{
    translateY: [1, 0]
  }],
  slideInDown: [{
    translateY: [-1, 0]
  }],
  slideOutLeft: [{
    exit: true,
    translateX: [0, -1]
  }],
  slideOutRight: [{
    exit: true,
    translateX: [0, 1]
  }],
  slideOutUp: [{
    exit: true,
    translateY: [0, -1]
  }],
  slideOutDown: [{
    exit: true,
    translateY: [0, 1]
  }],
  openSlidingDoors: [{
    exit: true,
    width: 0.5,
    translateX: [0, -0.5]
  }, {
    left: 0.5,
    width: 0.5,
    translateX: [0, 0.5]
  }],
  closeSlidingDoors: [{
    width: 0.5,
    translateX: [-0.5, 0]
  }, {
    left: 0.5,
    width: 0.5,
    translateX: [0.5, 0]
  }],
  openSlidingDoorsVertical: [{
    exit: true,
    height: 0.5,
    translateY: [0, -0.5]
  }, {
    top: 0.5,
    height: 0.5,
    translateY: [0, 0.5]
  }],
  closeSlidingDoorsVertical: [{
    height: 0.5,
    translateY: [-0.5, 0]
  }, {
    top: 0.5,
    height: 0.5,
    translateY: [0.5, 0]
  }],
  growCenter: [{
    scaleX: [0.001, 1],
    scaleY: [0.001, 1]
  }],
  shrinkCenter: [{
    exit: true,
    scaleX: [1, 0.001],
    scaleY: [1, 0.001]
  }],
  growTiles: [{
    width: 0.5,
    height: 0.5,
    scaleX: [0.001, 1],
    scaleY: [0.001, 1]
  }, {
    width: 0.5,
    height: 0.5,
    left: 0.5,
    scaleX: [0.001, 1],
    scaleY: [0.001, 1]
  }, {
    width: 0.5,
    height: 0.5,
    top: 0.5,
    scaleX: [0.001, 1],
    scaleY: [0.001, 1]
  }, {
    width: 0.5,
    height: 0.5,
    left: 0.5,
    top: 0.5,
    scaleX: [0.001, 1],
    scaleY: [0.001, 1]
  }]
  /*flipInX: [{ rotateX: ['90deg', '0deg'], overlayColor: 'black', overlayOpacity: [0.5, 0] }],
  flipOutX: [{ exit: true, rotateX: ['0deg', '90deg'] }],
  flipInY: [{ rotateY: ['90deg', '0deg'] }],
  flipOutY: [{ exit: true, rotateY: ['0deg', '90deg'] }],
  harmonicaInX: [
    {
      height: 0.5,
      translateY: a => mul(sub(1, arc2(a)), 0.25),
      //translateY: a => mul(arc2(a), 0.25),
      rotateX: ['90deg', '0deg'],
      overlayColor: 'white',
      overlayOpacity: [0.8, 0],
      // perspective: 10,
    },
    {
      height: 0.5,
      top: 0.5,
      translateY: a => mul(sub(1, arc2(a)), -0.25),
      //translateY: [-0.25, 0],
      rotateX: ['270deg', '360deg'],
      overlayColor: 'black',
      overlayOpacity: [0.8, 0],
      // perspective: 10,
    },
  ],*/

};
function resolveAnimation(animation) {
  if (typeof animation === 'string') {
    var dsl = ClippedAnimations[animation];
    if (!dsl) throw new Error("[Clipped] Invalid animation specified: ".concat(animation));
    animation = dsl;
  }

  if (!Array.isArray(animation)) {
    throw new Error("[Clipped] Invalid animation specified, not an array: ".concat(animation));
  }

  if (!animation.length) {
    throw new Error("[Clipped] Invalid animation specified, array is empty: ".concat(animation));
  }

  return animation;
}
function isExitAnimation(animation) {
  animation = resolveAnimation(animation);
  return animation[0].exit || false;
}

var ClippedView =
/*#__PURE__*/
function (_Component) {
  _inherits(ClippedView, _Component);

  function ClippedView(props) {
    var _this;

    _classCallCheck(this, ClippedView);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(ClippedView).call(this, props));

    _defineProperty(_assertThisInitialized(_this), "onLayout", function (event) {
      var _event$nativeEvent$la = event.nativeEvent.layout,
          height = _event$nativeEvent$la.height,
          width = _event$nativeEvent$la.width;

      if (_this.state.width !== width || _this.state.height !== height) {
        _this.setState({
          width: width,
          height: height
        });
      }
    });

    _this.state = {
      animation: undefined,
      animValue: undefined,
      anim: undefined,
      width: undefined,
      height: undefined
    };
    return _this;
  }

  _createClass(ClippedView, [{
    key: "renderFragments",
    value: function renderFragments(width, height, debug) {
      var _this$props = this.props,
          children = _this$props.children,
          _this$props$left = _this$props.left,
          left = _this$props$left === void 0 ? 0 : _this$props$left,
          _this$props$top = _this$props.top,
          top = _this$props$top === void 0 ? 0 : _this$props$top,
          _this$props$right = _this$props.right,
          right = _this$props$right === void 0 ? 0 : _this$props$right,
          _this$props$bottom = _this$props.bottom,
          bottom = _this$props$bottom === void 0 ? 0 : _this$props$bottom,
          _this$props$leftRotat = _this$props.leftRotate,
          leftRotate = _this$props$leftRotat === void 0 ? 0 : _this$props$leftRotat,
          _this$props$topRotate = _this$props.topRotate,
          topRotate = _this$props$topRotate === void 0 ? 0 : _this$props$topRotate,
          _this$props$rightRota = _this$props.rightRotate,
          rightRotate = _this$props$rightRota === void 0 ? 0 : _this$props$rightRota,
          _this$props$bottomRot = _this$props.bottomRotate,
          bottomRotate = _this$props$bottomRot === void 0 ? 0 : _this$props$bottomRot;
      var result = debug ? undefined : children;

      if (left && !leftRotate) {
        result = React__default.createElement(ClippedFragment, {
          debug: debug,
          width: width,
          height: height,
          translateX: reactNative.Animated.multiply(left, width)
        }, result);
      }

      if (leftRotate) {
        result = React__default.createElement(ClippedFragment, {
          debug: debug,
          width: width * 2,
          height: height * 2,
          originalWidth: width,
          originalHeight: height,
          top: (height * 2 - height) / -2,
          left: width / -2 //translateY={leftRotate * height}
          ,
          translateX: reactNative.Animated.add(width / 2, reactNative.Animated.multiply(left, width)),
          rotateZ: "".concat(leftRotate * 45, "deg")
        }, result);
      }

      if (right && !rightRotate) {
        result = React__default.createElement(ClippedFragment, {
          debug: debug,
          width: width,
          height: height,
          translateX: reactNative.Animated.multiply(right, -width)
        }, result);
      }

      if (rightRotate) {
        result = React__default.createElement(ClippedFragment, {
          debug: debug,
          width: width * 2,
          height: height * 2,
          originalWidth: width,
          originalHeight: height,
          top: (height * 2 - height) / -2,
          left: width / -2 //translateY={leftRotate * height}
          ,
          translateX: reactNative.Animated.add(width / -2, reactNative.Animated.multiply(right, -width)),
          rotateZ: "".concat(rightRotate * 45, "deg")
        }, result);
      }

      if (top && !topRotate) {
        result = React__default.createElement(ClippedFragment, {
          debug: debug,
          width: width,
          height: height,
          translateY: reactNative.Animated.multiply(top, height)
        }, result);
      }

      if (topRotate) {
        result = React__default.createElement(ClippedFragment, {
          debug: debug,
          width: width * 2,
          height: height * 2,
          originalWidth: width,
          originalHeight: height,
          top: height / -2,
          left: (width * 2 - width) / -2,
          translateY: reactNative.Animated.add(height / 2, reactNative.Animated.multiply(top, height)),
          rotateZ: "".concat(topRotate * 45, "deg")
        }, result);
      }

      if (bottom && !bottomRotate) {
        result = React__default.createElement(ClippedFragment, {
          debug: debug,
          width: width,
          height: height,
          translateY: reactNative.Animated.multiply(bottom, -height)
        }, result);
      }

      if (bottomRotate) {
        result = React__default.createElement(ClippedFragment, {
          debug: debug,
          width: width * 2,
          height: height * 2,
          originalWidth: width,
          originalHeight: height,
          top: height / -2,
          left: (width * 2 - width) / -2,
          translateY: reactNative.Animated.add(height / -2, reactNative.Animated.multiply(bottom, -height)),
          rotateZ: "".concat(bottomRotate * 45, "deg")
        }, result);
      }

      if (result === children) {
        result = React__default.createElement(reactNative.View, {
          style: reactNative.StyleSheet.absoluteFill
        }, children);
      }

      return result;
    }
  }, {
    key: "renderAnimation",
    value: function renderAnimation(width, height, debug) {
      // $FlowFixMe
      var animValue = this.state.animValue;
      var childContent = this.renderFragments(width, height, debug);
      var animation = resolveAnimation(this.state.animation); //const isExitAnimation = animation[0].exit || false;

      return animation.map(function (anim, idx) {
        var vals = {
          //move: this.props.move,
          debug: debug,
          left: 0,
          top: 0,
          width: width,
          height: height,
          originalWidth: width,
          originalHeight: height
        };
        /*if (fade) {
          vals.opacity = animValue.interpolate({
            inputRange: [0, 1],
            outputRange: isExitAnimation ? [1, 0] : [0, 1],
          });
        }*/

        Object.keys(anim).forEach(function (key) {
          var val = anim[key];
          var multiplier = 1;

          switch (key) {
            case 'left':
            case 'width':
            case 'translateX':
              multiplier = width;
              break;

            case 'top':
            case 'height':
            case 'translateY':
              multiplier = height;
              break;
          }

          if (Array.isArray(val)) {
            val = animValue.interpolate({
              inputRange: [0, 1],
              outputRange: val.map(function (arrVal) {
                return multiplier !== 1 ? arrVal * multiplier : arrVal;
              })
            });
          } else if (typeof val === 'function') {
            val = val(animValue);

            if (multiplier !== 1) {
              val = reactNative.Animated.multiply(val, multiplier);
            }
          } else if (multiplier !== 1) {
            val = val * multiplier;
          }

          vals[key] = val;
        });
        return React__default.createElement(ClippedFragment, _extends({
          key: idx
        }, vals), childContent);
      });
    }
  }, {
    key: "render",
    value: function render() {
      var _this$props2 = this.props,
          style = _this$props2.style,
          children = _this$props2.children,
          _this$props2$left = _this$props2.left,
          _this$props2$top = _this$props2.top,
          _this$props2$right = _this$props2.right,
          _this$props2$bottom = _this$props2.bottom,
          _this$props2$leftRota = _this$props2.leftRotate,
          _this$props2$topRotat = _this$props2.topRotate,
          _this$props2$rightRot = _this$props2.rightRotate,
          _this$props2$bottomRo = _this$props2.bottomRotate,
          animation = _this$props2.animation,
          duration = _this$props2.duration,
          delay = _this$props2.delay,
          easing = _this$props2.easing,
          useNativeDriver = _this$props2.useNativeDriver,
          onAnimationEnd = _this$props2.onAnimationEnd,
          debug = _this$props2.debug,
          otherProps = _objectWithoutProperties(_this$props2, ["style", "children", "left", "top", "right", "bottom", "leftRotate", "topRotate", "rightRotate", "bottomRotate", "animation", "duration", "delay", "easing", "useNativeDriver", "onAnimationEnd", "debug"]);

      var _this$state = this.state,
          width = _this$state.width,
          height = _this$state.height,
          anim = _this$state.anim;
      return React__default.createElement(reactNative.View, _extends({
        style: style,
        onLayout: this.onLayout
      }, otherProps), React__default.createElement(reactNative.View, {
        style: {
          opacity: 0
        },
        collapsable: false
      }, children), !anim && debug && width && height ? this.renderFragments(width, height, debug) : undefined, !anim && width && height ? this.renderFragments(width, height) : undefined, anim && debug && width && height ? this.renderAnimation(width, height, debug) : undefined, anim && width && height ? this.renderAnimation(width, height) : undefined);
    }
  }], [{
    key: "getDerivedStateFromProps",
    value: function getDerivedStateFromProps(props, state) {
      var newState = null;

      if (state.width === undefined || state.height === undefined) {
        return null;
      }

      if (!state.anim && props.animation) {
        newState = newState || {};
        var duration = props.duration,
            delay = props.delay,
            easing = props.easing,
            useNativeDriver = props.useNativeDriver;
        newState.animation = props.animation;
        newState.animValue = new reactNative.Animated.Value(0);
        newState.anim = reactNative.Animated.timing(newState.animValue, {
          toValue: 1,
          duration: duration,
          delay: delay,
          easing: easing,
          useNativeDriver: useNativeDriver
        });
        newState.anim.start(props.onAnimationEnd);
        /*} else if (state.anim && !props.animation) {
        newState = newState || {};
        newState.animation = undefined;
        newState.anim = undefined;
        newState.animValue = undefined;*/
      } else if (state.animation && props.animation && state.animation !== props.animation) {
        newState = newState || {};
        var _duration = props.duration,
            _delay = props.delay,
            _easing = props.easing,
            _useNativeDriver = props.useNativeDriver;
        newState.animation = props.animation;
        var animValue = new reactNative.Animated.Value(0);
        newState.animValue = reactNative.Animated.add(reactNative.Animated.subtract(state.animValue || 0, 1), animValue);
        newState.anim = reactNative.Animated.timing(animValue, {
          toValue: 1,
          duration: _duration,
          delay: _delay,
          easing: _easing,
          useNativeDriver: _useNativeDriver
        });
        newState.anim.start(props.onAnimationEnd);
      }

      return newState;
    }
  }]);

  return ClippedView;
}(React.Component);

_defineProperty(ClippedView, "defaultProps", {
  duration: 1000,
  delay: 0,
  useNativeDriver: true
});

var ClippedTransition =
/*#__PURE__*/
function (_Component) {
  _inherits(ClippedTransition, _Component);

  function ClippedTransition(props) {
    var _this;

    _classCallCheck(this, ClippedTransition);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(ClippedTransition).call(this, props));

    _defineProperty(_assertThisInitialized(_this), "onAnimationEnd", function () {
      var onTransitionEnd = _this.props.onTransitionEnd;

      if (_this.state.prevChildren.length) {
        _this.setState({
          prevChildren: []
        });
      }

      if (onTransitionEnd) onTransitionEnd();
    });

    _this.state = {
      width: undefined,
      height: undefined,
      child: React__default.Children.only(props.children),
      prevChildren: []
    };
    return _this;
  }

  _createClass(ClippedTransition, [{
    key: "render",
    value: function render() {
      var _this2 = this;

      var _this$props = this.props,
          style = _this$props.style,
          animation = _this$props.animation,
          hideAnimation = _this$props.hideAnimation,
          duration = _this$props.duration,
          delay = _this$props.delay,
          easing = _this$props.easing,
          useNativeDriver = _this$props.useNativeDriver,
          onTransitionEnd = _this$props.onTransitionEnd,
          debug = _this$props.debug,
          otherProps = _objectWithoutProperties(_this$props, ["style", "animation", "hideAnimation", "duration", "delay", "easing", "useNativeDriver", "onTransitionEnd", "debug"]);

      var _this$state = this.state,
          child = _this$state.child,
          prevChildren = _this$state.prevChildren;
      var children = child ? [].concat(_toConsumableArray(prevChildren), [child]) : prevChildren;
      var anim = resolveAnimation(animation);
      var hideAnim = hideAnimation ? resolveAnimation(hideAnimation) : undefined;
      return React__default.createElement(reactNative.View, _extends({
        style: style
      }, otherProps), React__default.Children.map(children, function (child) {
        var isHiding = child !== _this2.state.child;
        return React__default.createElement(ClippedView, {
          key: child.key,
          style: isHiding ? reactNative.StyleSheet.absoluteFill : undefined,
          animation: isHiding ? hideAnim : anim,
          duration: duration,
          delay: delay,
          easing: easing,
          useNativeDriver: useNativeDriver,
          onAnimationEnd: isHiding ? undefined : _this2.onAnimationEnd,
          debug: debug
        }, child);
      }));
    }
  }], [{
    key: "getDerivedStateFromProps",
    value: function getDerivedStateFromProps(props, state) {
      var newState = null;
      var child = React__default.Children.only(props.children);

      if (!child && state.child) {
        newState = newState || {};
        newState.child = undefined;
        newState.prevChildren = [].concat(_toConsumableArray(state.prevChildren), [state.child]);
      } else if (!state.child && child) {
        newState = newState || {};
        newState.child = child;
      } else if (state.child && child && state.child.key !== child.key) {
        newState = newState || {};
        newState.child = child;
        newState.prevChildren = [].concat(_toConsumableArray(state.prevChildren), [state.child]);
      }

      return newState;
    }
  }]);

  return ClippedTransition;
}(React.Component);

var ClippedLayouts = {
  rows: function rows(index, count) {
    return {
      left: index / count,
      right: (count - 1 - index) / count
    };
  },
  columns: function columns(index, count) {
    return {
      top: index / count,
      bottom: (count - 1 - index) / count
    };
  },
  rows2: function rows2(index, count) {
    return {
      left: index / count,
      leftRotate: 0.5,
      right: (count - 1 - index) / count,
      rightRotate: 0.5
    };
  }
};
function resolveLayout(layout) {
  if (typeof layout === 'string') {
    var dsl = ClippedLayouts[layout];
    if (!dsl) throw new Error("[Clipped] Invalid layout specified: ".concat(layout));
    layout = dsl;
  }

  if (!layout) {
    throw new Error("[Clipped] Invalid layout specified: ".concat(layout));
  }

  return layout;
}

var ClippedSeries =
/*#__PURE__*/
function (_Component) {
  _inherits(ClippedSeries, _Component);

  function ClippedSeries() {
    _classCallCheck(this, ClippedSeries);

    return _possibleConstructorReturn(this, _getPrototypeOf(ClippedSeries).apply(this, arguments));
  }

  _createClass(ClippedSeries, [{
    key: "render",
    value: function render() {
      var _this$props = this.props,
          style = _this$props.style,
          children = _this$props.children,
          layout = _this$props.layout,
          debug = _this$props.debug,
          otherProps = _objectWithoutProperties(_this$props, ["style", "children", "layout", "debug"]);

      var layoutFn = resolveLayout(layout);
      return React__default.createElement(reactNative.View, _extends({
        style: style
      }, otherProps), React__default.Children.map(children, function (child, index) {
        return React__default.createElement(ClippedView, _extends({
          key: index,
          style: index ? reactNative.StyleSheet.absoluteFill : undefined,
          debug: debug
        }, layoutFn(index, React__default.Children.count(children))), child);
      }));
    }
  }]);

  return ClippedSeries;
}(React.Component);

var Fragment = ClippedFragment;
var View = ClippedView;
var Transition = ClippedTransition;
var Series = ClippedSeries;
var Animations = Object.keys(ClippedAnimations);
var Layouts = Object.keys(ClippedLayouts);
var isExitAnimation$1 = isExitAnimation;

exports.Fragment = Fragment;
exports.View = View;
exports.Transition = Transition;
exports.Series = Series;
exports.Animations = Animations;
exports.Layouts = Layouts;
exports.isExitAnimation = isExitAnimation$1;
